@@ -200,8 +200,12 @@ int sig;
 	}
 #if 0
 	/* if we are doing an i_read, restart it */
-	if (env_valid && (sig != 0)) longjmp(env,2);
-#endif
+#ifdef HAVE_SIGLONGJMP
+      if (env_valid && (sig != 0)) siglongjmp(env,2);
+#else
+      if (env_valid && (sig != 0)) longjmp(env,2);
+#endif  /* HAVE_SIGLONGJMP */
+#endif /* 0 */
 }
 
 /*ARGSUSED*/
@@ -370,9 +374,7 @@ Tcl_Obj *CONST objv[];
 
 	/* objv[1] is the list of signals - crack it open */
 	if (TCL_OK != Tcl_ListObjGetElements(interp,objv[1],&n,&list)) {
-		expErrorLogU(Tcl_GetStringResult(interp));
-		expErrorLogU("\r\n");
-		goto usage_error;
+	  return TCL_ERROR;
 	}
 
 	for (i=0;i<n;i++) {
@@ -470,8 +472,6 @@ int oldcode;
 		expDiagLogU(trap->action);
 		expDiagLogU("\r\n");
 		if (0 != strcmp(Tcl_GetStringResult(interp),"")) {
-			expErrorLogU(interp->result);
-			expErrorLogU("\r\n");
 
 			/*
 			 * Check errorinfo and see if it contains -nostack.
