@@ -517,6 +517,7 @@ Tcl_Obj *CONST objv[];		/* Argument objects. */
 		ec.i_list = exp_new_i_complex(interp,
 				      Tcl_GetString(objv[i]),
 				      eg->duration, exp_indirect_update2);
+		if (!ec.i_list) goto error;
 		ec.i_list->cmdtype = eg->cmdtype;
 
 		/* link new i_list to head of list */
@@ -853,7 +854,8 @@ char *suffix;
 	    return EXP_MATCH;
 	}
 	expDiagLogU(no);
-    } else if ((Tcl_GetCharLength(esPtr->buffer) == esPtr->msize)
+    } else if (e->use == PAT_FULLBUFFER) {
+      if ((Tcl_GetCharLength(esPtr->buffer) == esPtr->msize)
 	    && (length > 0)) {
 	expDiagLogU(Tcl_GetString(e->pat));
 	expDiagLogU("? ");
@@ -863,6 +865,9 @@ char *suffix;
 	o->esPtr = esPtr;
 	expDiagLogU(yes);
 	return(EXP_FULLBUFFER);
+      } else {
+	expDiagLogU(no);
+      }
     }
     return(EXP_NOMATCH);
 }
